//////////////////////////////////////////////////////////////////////////////////
// 
// Create Date:   01/23/2015
// Revised:			08/11/2015
// Module Name:   colors_12 
// Project Name: 	c3000
// Description:	module takes an number as input and uses it to index into a 
// 					table to obtain a 12 bit color for output. 
// 					12 bit colors used for text, background, border, LORES, 
// 					double HIRES & double LORES
// 
//////////////////////////////////////////////////////////////////////////////////
`include "c3000_defs.v"
module colors_12(
	 input monochrome,				// used with double hires
	 input [7:0] CBVID,
    input [3:0] border_number,
	 input [3:0] text_number,
	 input [3:0] back_number,
	 input [3:0] dhrcolor_number,
	 input [3:0] nibble,
	 input [3:0] dlr_nibble,
    output reg [11:0] border_color12,
    output reg [11:0] text_color12,	 
    output reg [11:0] back_color12,
    output reg [11:0] dhr_color12,
	 output reg [11:0] lowres_color_val12,
	 output reg [11:0] dlr_color12
    );
//////////////////////////////////////////////////////////////////////////////////
// 12 bit colors used 
/*
Color    Color Register    Master Color	
Name         Value            Value			Bits	 
------------------------------------------------
Black         $0              $0000			0000
Deep Red      $1              $0D03			0001
Dark Blue     $2              $001A		 	0010	  
Purple        $3              $0D2D		 	0011
Dark Green    $4              $0072			0100
Dark Gray     $5              $0555			0101
Medium Blue   $6              $001F		   0110	
Light Blue    $7              $049F		 	0111		
Brown         $8              $0850			1000	
Orange        $9              $0F60			1001
Light Gray    $A              $0AAA			1010
Pink          $B              $0F98		 	1011	
Light Green   $C              $01D4			1100			
Yellow        $D              $0FF0			1101			
Aquamarine    $E              $04F9			1110		
White         $F              $0FFF			1111
------------------------------------------------
*/
// border color
	always @* begin
		case (border_number)
			4'b0000: border_color12 = 12'h000;
			4'b0001: border_color12 = 12'hD03;
			4'b0010: border_color12 = 12'h01A;  
			4'b0011: border_color12 = 12'hD2D;
			4'b0100: border_color12 = 12'h072;
			4'b0101: border_color12 = 12'h555;
			4'b0110: border_color12 = 12'h01F;  
			4'b0111: border_color12 = 12'h49F;	
			4'b1000: border_color12 = 12'h850;
			4'b1001: border_color12 = 12'hF60;
			4'b1010: border_color12 = 12'hAAA;
			4'b1011: border_color12 = 12'hF98;
			4'b1100: border_color12 = 12'h1D4;		
			4'b1101: border_color12 = 12'hFF0;
			4'b1110: border_color12 = 12'h4F9;
			4'b1111: border_color12 = 12'hFFF;			
		endcase
	end
// text color
	always @* begin
		case (text_number)
			4'b0000: text_color12 = 12'h000;
			4'b0001: text_color12 = 12'hD03;
			4'b0010: text_color12 = 12'h01A;  
			4'b0011: text_color12 = 12'hD2D;
			4'b0100: text_color12 = 12'h072;
			4'b0101: text_color12 = 12'h555;
			4'b0110: text_color12 = 12'h01F;  
			4'b0111: text_color12 = 12'h49F;	
			4'b1000: text_color12 = 12'h850;
			4'b1001: text_color12 = 12'hF60;
			4'b1010: text_color12 = 12'hAAA;
			4'b1011: text_color12 = 12'hF98;
			4'b1100: text_color12 = 12'h1D4;	
			4'b1101: text_color12 = 12'hFF0;
			4'b1110: text_color12 = 12'h4F9;
			4'b1111: text_color12 = 12'hFFF;			
		endcase
	end	
// background color	
	always @* begin
		case (back_number)
			4'b0000: back_color12 = 12'h000;
			4'b0001: back_color12 = 12'hD03;
			4'b0010: back_color12 = 12'h01A; 
			4'b0011: back_color12 = 12'hD2D;
			4'b0100: back_color12 = 12'h072;
			4'b0101: back_color12 = 12'h555;
			4'b0110: back_color12 = 12'h01F;  
			4'b0111: back_color12 = 12'h49F;	 
			4'b1000: back_color12 = 12'h850;
			4'b1001: back_color12 = 12'hF60;
			4'b1010: back_color12 = 12'hAAA;
			4'b1011: back_color12 = 12'hF98;
			4'b1100: back_color12 = 12'h1D4;		
			4'b1101: back_color12 = 12'hFF0;
			4'b1110: back_color12 = 12'h4F9;
			4'b1111: back_color12 = 12'hFFF;			
		endcase
	end
// double HIRES color
	always @* 
		begin
		if (monochrome)
			begin
			dhr_color12 = {dhrcolor_number, dhrcolor_number, dhrcolor_number};
			end
		else
			begin
			case (dhrcolor_number)
			4'b0000: dhr_color12 = 12'h000;	// black
			4'b0001: dhr_color12 = 12'hD03;	// deep red
			4'b0010: dhr_color12 = 12'h850;  // brown 
			4'b0011: dhr_color12 = 12'hF60;	// orange 
			4'b0100: dhr_color12 = 12'h072;	// dark green
			4'b0101: dhr_color12 = 12'h555;	// dark gray
			4'b0110: dhr_color12 = 12'h1D4;  // light green (or just green)
			4'b0111: dhr_color12 = 12'hFF0;	// yellow 
			4'b1000: dhr_color12 = 12'h01A;	// dark blue
			4'b1001: dhr_color12 = 12'hD2D;	// purple
			4'b1010:
				begin
				if (~CBVID[1]) dhr_color12 = 12'h555; // gray  (on a IIgs 12'hAAA;)
				else  dhr_color12 = 12'hAAA; 	// light gray
				end
			4'b1011: dhr_color12 = 12'hF98;	// pink
			4'b1100: dhr_color12 = 12'h01F;	// medium blue		
			4'b1101: dhr_color12 = 12'h49F;	// light blue
			4'b1110: dhr_color12 = 12'h4F9;	// aquamarine
			4'b1111: dhr_color12 = 12'hFFF;	// white		
			endcase
			end
		end
// double LORES color	
	always @* begin
		case (dlr_nibble)
			4'b0000: dlr_color12 = 12'h000;	// black
			4'b0001: dlr_color12 = 12'hD03;	// deep red
			4'b0010: dlr_color12 = 12'h850;  // brown 
			4'b0011: dlr_color12 = 12'hF60;	// orange
			4'b0100: dlr_color12 = 12'h072;	// dark green
			4'b0101: dlr_color12 = 12'h555;	// dark gray
			4'b0110: dlr_color12 = 12'h1D4;  // light green (or just green)
			4'b0111: dlr_color12 = 12'hFF0;	// yellow 
			4'b1000: dlr_color12 = 12'h01A;	// dark blue
			4'b1001: dlr_color12 = 12'hD2D;	// purple
			4'b1010: 
				begin
				if (~CBVID[1]) dlr_color12 = 12'h555; // gray  (on a IIgs 12'hAAA;)
				else  dlr_color12 = 12'hAAA; 	// light gray
				end
			4'b1011: dlr_color12 = 12'hF98;	// pink
			4'b1100: dlr_color12 = 12'h01F;	// medium blue		
			4'b1101: dlr_color12 = 12'h49F;	// light blue
			4'b1110: dlr_color12 = 12'h4F9;	// aquamarine
			4'b1111: dlr_color12 = 12'hFFF;	// white		
		endcase
	end
// LORES color	
	always @* begin
		case (nibble)
			4'b0000: lowres_color_val12 = 12'h000;	// black
			4'b0001: lowres_color_val12 = 12'hD03;	// deep red
			4'b0010: lowres_color_val12 = 12'h01A;  
			4'b0011: lowres_color_val12 = 12'hD2D;
			4'b0100: lowres_color_val12 = 12'h072;
			4'b0101: lowres_color_val12 = 12'h555;	// gray  (on a IIgs 12'h555;)
			4'b0110: lowres_color_val12 = 12'h01F;  
			4'b0111: lowres_color_val12 = 12'h49F;	 
			4'b1000: lowres_color_val12 = 12'h850; // brown 
			4'b1001: lowres_color_val12 = 12'hF60;	// orange
			4'b1010: 
				begin
				if (~CBVID[1]) lowres_color_val12 = 12'h555; // gray  (on a IIgs 12'hAAA;)
				else  lowres_color_val12 = 12'hAAA;	// light gray
				end
			4'b1011: lowres_color_val12 = 12'hF98;
			4'b1100: lowres_color_val12 = 12'h1D4;		
			4'b1101: lowres_color_val12 = 12'hFF0;
			4'b1110: lowres_color_val12 = 12'h4F9;
			4'b1111: lowres_color_val12 = 12'hFFF;			
		endcase
	end		
endmodule
