////////////////////////////////////////////////////////////////////////////////
// c3000 project
//////////////////////////////////////////////////////////////////////////////////
// this module is a slightly modified version of the original from JAT on the CB1
`include "c3000_defs.v"
				
module write_clk(
                 input clk,					// 50 MHz master clock
                 input ph0,					// phase 0 clock from Apple
                 input Q3,						// Q3 clock from Apple
                 input rw,						// read / write signal from Apple
                 input update_time,			// when it's okay to get data (from vga_video.v)
                 input [15:0] bus_a_s,		// sync'd in cb2.v
                 input [7:0] bus_d_s,		// sync'd in cb2.v
                 output we,					// write enable for CB RAM
                 output video_write_SRAM,	// when we have something to write to CB RAM
                 output [15:0] ram_addr,	// RAM address to write
                 output [7:0] write_data	// RAM data to write
                 );

   reg 	Q3_reg = 0;
   reg 	Q3_reg1 = 0;

   reg [15:0] 	pending_addr = 0;
   reg [7:0] 	pending_data = 0;
   reg ph0_r1 = 0;
   reg ph0_r2 = 0;
   reg [2:0] 	state = 3'b000;  
   reg [2:0] 	next_state = 3'b000;
   reg write_from_6502;
	reg rw_1 = 0;
	reg rw_s = 0;
	wire wr_cl = ~Q3_reg1 && Q3_reg && ph0_r2;	// falling edge of Q3 and phase 0 (delayed) high
	
   always @ (posedge clk) 
		begin
      Q3_reg1 <= Q3; 		// sync with the 50 MHz clock
      Q3_reg <= Q3_reg1;	
      ph0_r1 <= ph0;
      ph0_r2 <= ph0_r1;
		rw_1 <= rw;
		rw_s <= rw_1;
      write_from_6502 <= ((wr_cl == 1) & (rw_s == 0)); 
      state <= next_state;
      if(write_from_6502) 
			begin
			pending_addr <= bus_a_s;
			pending_data <= bus_d_s;
			end		
		end

   always @(*)	
		begin
      case (state)
			3'h0:
				if(write_from_6502)
				next_state <= 1;
				else
				next_state <= 0;
			3'h1:
				if(update_time) // too late, wait for the next slot
				next_state <= 1;
				else
				next_state <= 2;
			3'h2:
				if(update_time)
				next_state <= 3;
				else
				next_state <= 2;
			3'h3:
				next_state <= 4;
			3'h4:
				next_state <= 5;
			3'h5:	
				next_state <= 0;
			default:
				next_state <= 0;
      endcase
		end // always @ (*)

	assign we = (state == 4);								
	assign video_write_SRAM = (state == 3) || (state == 4); // removed state == 5 to remove flickers from SHR
   assign write_data = pending_data;
   assign ram_addr = pending_addr;

endmodule